;; scilab-mode.el - major mode for editing Scilab source with GNU Emacs
;;
;; This major mode for GNU Emacs provides support for editing Scilab
;; source files.  It automatically indents for block structures, line
;; continuations (e.g., ...), and comments.  The usual paren matching
;; support is included.  
;; For convient use add something like the
;; following to your .emacs start-up file:
;;
;; (global-font-lock-mode t)
;; (load "scilab")
;; (setq auto-mode-alist (cons '("\(\.sci$\|\.sce$\)" . scilab-mode) auto-mode-alist))
;; (setq scilab-mode-hook '(lambda () (setq fill-column 74)))
;;
;;
;; Enjoy.
;;
;; Last modified Tue Oct 28 12:05:36 MET 1997
;;
;; This file was modified by Serge Steer (Serge.Steer@inria.fr ) from
;; the file octave-mode.el (Copyright (C) 1991 Matthew R. Wette.), 
;; fortran-mode.el Copyright (c) Free Software Foundation, Inc. (string handling)
;; tex-mode.el Copyright (C) Free Software Foundation, Inc. (compilation part) 
;; 
;; This file is modifed by Rainer Menzner (Rainer.Menzner@neuroinformatik.ruhr-uni-bochum.de)
;; to add font-lock support to Scilab-mode instead of hilit19.
;;
;; Everyone is granted permission to copy, modify and redistribute this
;; file provided:
;;   1. All copies contain this copyright notice.
;;   2. All modified copies shall carry a prominant notice stating who
;;      made the last modification and the date of such modification.
;;   3. No charge is made for this software or works derived from it.
;;      This clause shall not be construed as constraining other software
;;      distributed on the same medium as this software, nor is a
;;      distribution fee considered a charge.
;;

(require 'shell)
(require 'compile)
 
;; Constants used in all Scilab-mode buffers.
(defconst scilab-indent-level 2
  "*The indentation in Scilab-mode.")

(defconst scilab-comment-column 40
  "*The goal comment column in Scilab-mode buffers.")

(defconst sci-block-beg-kw "\(for[ \t]\|while[ \t]\|if[ \t]\|else[ \t,;\n]*\|elseif[ \t]\|select[ \t]\|case[ \t]\)"
  "Regular expression for keywords which begin blocks in Scilab-mode.")

(defconst sci-block-end-kw "\(end[ \t,;\n]*\|else[ \t,;\n]*\|elseif[ \t]\|case[ \t]\)"
  "Regular expression for keywords which end blocks.")

;; variables for compilation part
(defvar sci-last-indent-type "unknown"
  "String to tell line type.")

(defvar scilab-comment-region "//"
  "*String inserted by \[scilab-comment-region] at start of each line in region.")

;;;###autoload
(defvar scilab-command "scilab"
  "Command to run Scilab. scilab must be in the path.
The option and the name of the file, will be added to this string.")

;;;###autoload
(defvar scilab-last-buffer-compiled nil
  "Buffer which was last compiled.")
;;;###autoload
(defvar scilab-zap-file nil
  "Temporary file name used for text being sent as input to Scilab.
Should be a simple file name with no extension or directory specification."
)
;;;###autoload
(defvar scilab-directory "."
  "*Directory in which temporary files are left.
You can make this /tmp if your functions has no relative directories in it.")

;;;###autoload
(defvar scilab-offer-save t
  "*If non-nil, ask about saving modified buffers before \[scilab-file] is run.")
;;;###autoload
(defvar scilab-last-temp-file nil
  "Latest temporary file generated by \[scilab-region] and \[scilab-compile-buffer].
Deleted when the \[scilab-region] or \[scilab-compile-buffer] is next run, or when the
scilab-shell goes away.")

;;;###autoload
(defvar scilab-trailer nil
  "String appended after the end of a region sent to scilab by \[scilab-region].")

(defvar scilab-shell-file-name nil
  "*If non-nil, the shell file name to run in the subshell used to run Scilab.")

;; Syntax Table
(defvar scilab-mode-syntax-table nil
  "Syntax table used in Scilab-mode buffers.")


(if scilab-mode-syntax-table
    ()
  (setq scilab-mode-syntax-table (make-syntax-table))
  (modify-syntax-entry ?\ "." scilab-mode-syntax-table)
  (modify-syntax-entry ?/ "." scilab-mode-syntax-table)
  (modify-syntax-entry ?* "." scilab-mode-syntax-table)
  (modify-syntax-entry ?+ "." scilab-mode-syntax-table)
  (modify-syntax-entry ?- "." scilab-mode-syntax-table)
  (modify-syntax-entry ?= "." scilab-mode-syntax-table)
  (modify-syntax-entry ?< "." scilab-mode-syntax-table)
  (modify-syntax-entry ?> "." scilab-mode-syntax-table)
  (modify-syntax-entry ?\[ "(]" scilab-mode-syntax-table)
  (modify-syntax-entry ?\] ")[" scilab-mode-syntax-table)
  (modify-syntax-entry ?& "." scilab-mode-syntax-table)
  (modify-syntax-entry ?| "." scilab-mode-syntax-table)
  (modify-syntax-entry ?\' "'" scilab-mode-syntax-table)
  (modify-syntax-entry ?\n ">" scilab-mode-syntax-table)
  (set-syntax-table scilab-mode-syntax-table))


;; Abbrev Table
(defvar scilab-mode-abbrev-table nil
  "Abbrev table used in Scilab-mode buffers.")

(define-abbrev-table 'scilab-mode-abbrev-table ())


;; Mode Map
;(defvar scilab-mode-map nil "Keymap for Scilab mode.")
(defvar scilab-mode-map ()   "Keymap used in scilab-mode.")

(if scilab-mode-map
    nil
  (setq scilab-mode-map (make-sparse-keymap))
  (define-key scilab-mode-map "\r" 'scilab-return)
  (define-key scilab-mode-map "\t" 'scilab-indent-line)
  (define-key scilab-mode-map "\M-;" 'scilab-comment)
  (define-key scilab-mode-map "\C-ct" 'scilab-line-type)
  (define-key scilab-mode-map "\C-ci" 'scilab-indent-type)
  (define-key scilab-mode-map "\C-c\C-p" 'beginning-of-scilab-function)
  (define-key scilab-mode-map "\C-c\C-n" 'end-of-scilab-function)
  (define-key scilab-mode-map "\M-\r" 'newline))
  
(defvar menu-bar-scilab-menu (make-sparse-keymap "Scilab"))
(define-key scilab-mode-map [menu-bar scilab] (cons "Scilab" menu-bar-scilab-menu))
;(define-key menu-bar-scilab-menu [C2]
;  '("Check Buffer Syntax" . scilab-compile-buffer))
;(define-key menu-bar-scilab-menu [C3]
;  '("Check Function Syntax" . scilab-compile-function))
(define-key menu-bar-scilab-menu [C4]
  '("Indent Function" . scilab-indent-function))
(define-key menu-bar-scilab-menu [C5]
  '("Indent Buffer" . scilab-indent-buffer))
(define-key menu-bar-scilab-menu [C6]
  '("Ident Region" . scilab-indent-region))
(define-key menu-bar-scilab-menu [end-of-scilab-function]
  '("End of function C-cC-n" . end-of-scilab-function))
(define-key menu-bar-scilab-menu [beginning-of-scilab-function]
  '("Beginning of function C-cC-p" . beginning-of-scilab-function))



  (define-key scilab-mode-map [menu-bar scilab scilab-compile] 
    '(cons "Scilab Compile" scilab-compile))
 (define-key scilab-mode-map [menu-bar scilab scilab-compile-buffer] 
    '(cons "Scilab Compile Buffer" scilab-compile-buffer))
  (define-key scilab-mode-map [menu-bar scilab scilab-compile-function] 
    '(cons "Scilab Compile Function" scilab-compile-function))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Scilab font-lock support (RMz, 10/28/1997)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; emacs-20:  keyword builtin warning variable-name type string reference function-name comment
;; emacs-19:  comment string keyword function-name variable-name type reference
(defvar scilab-font-lock-keywords
  '(
    ("//.*$" . font-lock-comment-face)
    ("['\"][^'\n]*['\"]" . font-lock-string-face)
    ("[ ,;\t]\(do\|if\|elseif\|then\|else\|for\|end\|while\|select\|case\|break\)[ \t\n]" . font-lock-keyword-face)
    ("^\(do\|if\|elseif\|then\|else\|for\|end\|while\|select\|case\|break\)[ ,;\t\n]" . font-lock-keyword-face)
    ("[ ,;\t]\(resume\|return\|pause\|quit\|abort\|^//end\)[ ,;\t\n]" . font-lock-type-face)
    ("^ *\(endfunction\|function\)[ \t\n]" . font-lock-function-name-face)
    ("[^ ,;\t]\case\[ \t\n]" . label)
    )
  "Default expressions to highlight in Scilab mode."
  )



;; Scilab Mode
(defun scilab-mode ()
  "Major mode for editing Scilab source files.  Version 1.0, may 1997.
Will run scilab-mode-hook if it is non-nil.  Auto-fill-mode seems to work.
Filling does not work (yet).
Special Key Bindings:
\{scilab-mode-map}
Variables:
  scilab-indent-level                   Level to indent blocks.
  scilab-comment-column                 Goal column for on-line comments.
  fil
